@@ -16,6 +16,7 @@ try:
     from . import screens
     from . import command
     from . import exceptions
+    from . import version
 except ModuleNotFoundError:
     import data_type
     import i18n
@@ -23,8 +24,9 @@ except ModuleNotFoundError:
     import screens
     import command
     import exceptions
+    import version
 
-# new_event_loop = list()
+websockets.http.USER_AGENT += f' PyPtt/{version.V}'
 
 
 class connect_mode(object):
@@ -179,7 +181,6 @@ class API(object):
             ],
             i18n.Active)
 
-        telnet_host, websocket_host, websocket_origin = '', '', ''
         if self.config.host == data_type.host_type.PTT1:
             telnet_host = 'ptt.cc'
             websocket_host = 'wss://ws.ptt.cc/bbs/'
@@ -188,10 +189,14 @@ class API(object):
             telnet_host = 'ptt2.cc'
             websocket_host = 'wss://ws.ptt2.cc/bbs/'
             websocket_origin = 'https://term.ptt2.cc'
-        else:
+        elif self.config.host == data_type.host_type.LOCALHOST:
             telnet_host = 'localhost'
             websocket_host = 'wss://localhost'
             websocket_origin = 'https://term.ptt.cc'
+        else:
+            telnet_host = self.config.host
+            websocket_host = f'wss://{self.config.host}'
+            websocket_origin = 'https://term.ptt.cc'
 
         if self.config.connect_mode == connect_mode.TELNET:
             log.show_value(
@@ -208,9 +213,6 @@ class API(object):
 
         connect_success = False
 
-        # global new_event_loop
-        # thread_id = threading.get_ident()
-
         for _ in range(2):
 
             try:
@@ -220,13 +222,15 @@ class API(object):
 
                 else:
 
-                    # if thread_id not in new_event_loop:
-                    #     new_event_loop.append(thread_id)
-                    try:
+                    if not threading.current_thread() is threading.main_thread():
                         loop = asyncio.new_event_loop()
                         asyncio.set_event_loop(loop)
-                    except:
-                        pass
+
+                    log.show_value(
+                        self.config,
+                        log.level.DEBUG,
+                        'USER_AGENT',
+                        websockets.http.USER_AGENT)
 
                     self._core = asyncio.get_event_loop().run_until_complete(
                         websockets.connect(
@@ -259,6 +263,135 @@ class API(object):
         if not connect_success:
             raise exceptions.ConnectError(self.config)
 
+    def fast_send(
+            self,
+            msg: str,
+            target_list: list,
+            refresh: bool = True) -> int:
+
+        break_detect_after_send = False
+        break_index = -1
+
+        use_too_many_res = False
+        while True:
+
+            if refresh and not msg.endswith(command.Refresh):
+                msg = msg + command.Refresh
+            try:
+                msg = msg.encode('big5uao', 'replace')
+
+            except AttributeError:
+                pass
+            except Exception as e:
+                traceback.print_tb(e.__traceback__)
+                print(e)
+                msg = msg.encode('big5', 'replace')
+
+            try:
+                asyncio.get_event_loop().run_until_complete(
+                    self._core.send(msg))
+            except websockets.exceptions.ConnectionClosedError:
+                raise exceptions.ConnectionClosed()
+            except RuntimeError:
+                raise exceptions.ConnectionClosed()
+            except websockets.exceptions.ConnectionClosedOK:
+                raise exceptions.ConnectionClosed()
+
+            if break_detect_after_send:
+                return break_index
+
+            msg = ''
+            receive_data_buffer = bytes()
+
+            # print(f'0 {use_too_many_res}')
+            start_time = time.time()
+            mid_time = time.time()
+            while mid_time - start_time < self.config.screen_timeout:
+
+                recv_data_obj = RecvData()
+
+                try:
+
+                    asyncio.get_event_loop().run_until_complete(
+                        websocket_receiver(
+                            self._core, self.config.screen_timeout, recv_data_obj))
+
+                except websockets.exceptions.ConnectionClosed:
+                    # print(f'0.1 {use_too_many_res}')
+                    if use_too_many_res:
+                        # print(f'0.2 {use_too_many_res}')
+                        raise exceptions.UseTooManyResources()
+                    # print(f'0.3 {use_too_many_res}')
+                    raise exceptions.ConnectionClosed()
+                except websockets.exceptions.ConnectionClosedOK:
+                    raise exceptions.ConnectionClosed()
+                except asyncio.TimeoutError:
+                    return -1
+                except RuntimeError:
+                    raise exceptions.ConnectionClosed()
+
+                receive_data_buffer += recv_data_obj.data
+                screen = receive_data_buffer.decode(
+                    'big5uao', errors='replace')
+
+                find_target = False
+                for Target in target_list:
+                    condition = Target.is_match(screen)
+                    if condition:
+                        if len(screen) > 0:
+                            screens.show(self.config, screen)
+                            self._RDQ.add(screen)
+                            # self._ReceiveDataQueue.append(screen)
+                            if Target == self._UseTooManyResources:
+                                # print('!!!!!!!!!!!!!!!')
+                                use_too_many_res = True
+                                # print(f'1 {use_too_many_res}')
+                                break
+                            Target.raise_exception()
+
+                        find_target = True
+
+                        if Target.is_break():
+                            return target_list.index(Target)
+
+                        msg = Target.get_response(screen)
+
+                        add_refresh = False
+                        if Target.is_refresh():
+                            add_refresh = True
+                        elif refresh:
+                            add_refresh = True
+
+                        if add_refresh:
+                            if not msg.endswith(command.Refresh):
+                                msg = msg + command.Refresh
+
+                        if Target.is_break_after_send():
+                            break_index = target_list.index(Target)
+                            break_detect_after_send = True
+                        break
+
+                # print(f'2 {use_too_many_res}')
+                if use_too_many_res:
+                    # print(f'3 {use_too_many_res}')
+                    continue
+                # print(f'4 {use_too_many_res}')
+
+                if find_target:
+                    break
+                if len(screen) > 0:
+                    screens.show(self.config, screen)
+                    self._RDQ.add(screen)
+                    # self._ReceiveDataQueue.append(screen)
+
+                mid_time = time.time()
+
+            if not find_target:
+                # raise exceptions.NoMatchTargetError(self._RDQ)
+                return -1
+        # raise exceptions.NoMatchTargetError(self._RDQ)
+        return -1
+
     def send(
             self,
             msg: str,
@@ -311,7 +444,7 @@ class API(object):
             if refresh and not msg.endswith(command.Refresh):
                 msg = msg + command.Refresh
             try:
-                msg = msg.encode('big5-uao', 'replace')
+                msg = msg.encode('big5uao', 'replace')
 
             except AttributeError:
                 pass
@@ -393,9 +526,19 @@ class API(object):
 
                 receive_data_buffer += recv_data_obj.data
                 receive_data_temp = receive_data_buffer.decode(
-                    'big5-uao', errors='replace')
+                    'big5uao', errors='replace')
                 screen = clean_screen(receive_data_temp)
 
+                # qq = receive_data_temp.encode('utf-8')
+                # b = None
+                # for q in qq:
+                #     if not b:
+                #         b = f'bytes([{q}'
+                #     else:
+                #         b += f', {q}'
+                # b += f'])'
+                # print(b)
+
                 find_target = False
                 for Target in target_list:
                     condition = Target.is_match(screen)
